@@ -29,16 +29,22 @@ class Action:
         self.port = port or jconfig.port
         self.address = utils.to_address(self.host, self.port)
 
-        self.qq = int(qq or jconfig.qq)  # type: ignore
+        self._qq = int(qq or jconfig.qq or 0)
 
         self.c = httpx.Client(
             headers={"Content-Type": "application/json"},
             timeout=timeout + 5,
             base_url=self.address,
-            params={"qq": self.qq, "timeout": timeout},
+            params={"timeout": timeout},
         )
         self.lock = threading.Lock()
 
+    @property
+    def qq(self) -> int:
+        if self._qq == 0:
+            self._qq = self.getAllBots()[0]
+        return self._qq
+
     @classmethod
     def from_ctx(
         cls, ctx: Union[EventMsg, FriendMsg, GroupMsg], timeout: int = 20
@@ -641,7 +647,7 @@ class Action:
 
     def getClusterInfo(self) -> dict:
         """获取当前集群信息"""
-        return self.get("", path="/v1/ClusterInfo")
+        return self.get("", path="/v1/ClusterInfo", params={"isShow": 1, "qq": 1})
 
     ############操作############
     def setUniqueTitle(self, user: int, group: int, title: str) -> dict:
@@ -778,6 +784,28 @@ class Action:
         """刷新key二次登陆"""
         return self.get("", path="/v1/RefreshKeys")
 
+    def dealFriend(self, ctx: EventMsg, cmd=None) -> dict:
+        """处理好友请求
+        :param ctx: 事件EventMsg, 类型不匹配将报错
+        :param cmd: True:同意,False:拒绝,None:忽略
+        """
+        friend_add_info = eventParser.friend_add(ctx)
+        assert friend_add_info, "事件类型不匹配"
+        return self.post(
+            "DealFriend",
+            {
+                "UserID": friend_add_info.UserID,
+                "FromType": friend_add_info.FromType,
+                "Type": friend_add_info.Type,
+                "Field_3": friend_add_info.Field_3,
+                "Field_8": friend_add_info.Field_8,
+                "Content": friend_add_info.Content,
+                "FromGroupId": friend_add_info.FromGroupId,
+                "FromGroupName": friend_add_info.FromGroupName,
+                "Action": {True: 2, False: 3, None: 1}[cmd],  # 1忽略2同意3拒绝
+            },
+        )
+
     def logout(self, flag=False) -> dict:
         """退出指定QQ
         :param flag: 是否删除设备信息文件
@@ -800,6 +828,10 @@ class Action:
         """
         return self.post("", {"HDIMGUrl": url}, path="/v1/SelfHDIMG")
 
+    def getAllBots(self) -> List[int]:
+        """获取OPQ登陆的所有机器人QQ号"""
+        return [i["QQ"] for i in self.getClusterInfo()["QQUsers"]]
+
     ############################################################################
     def baseRequest(
         self,
@@ -810,10 +842,10 @@ class Action:
         params: Optional[dict] = None,
     ) -> dict:
         """基础请求方法, 提供部分提示信息，出错返回空字典，其他返回服务端响应结果"""
-        if params is not None:
-            params.update({"funcname": funcname})
-        else:
-            params = {"funcname": funcname}
+        params = params or {}
+        params["funcname"] = funcname
+        if not params.get("qq"):
+            params["qq"] = self.qq
 
         # 发送请求
         try:
@@ -828,7 +860,7 @@ class Action:
             return {}
         except httpx.HTTPStatusError:
             logger.error(
-                f"响应码出错 => {resp.status_code}",  # type:ignore
+                f"响应码出错 => {resp.status_code}，大概率是因为账号已离线或者qq号错误",  # type:ignore
             )
             return {}
         except Exception:
